<details>
  <summary>Imports</summary>

```
module HottBook.Chapter2 where

open import Agda.Primitive
open import HottBook.Chapter1
open import HottBook.Chapter2Lemma221
open import HottBook.Util
```

</details>

# 2 Homotopy type theory

## 2.1 Types are higher groupoids

### Lemma 2.1.1

```
sym : {l : Level} {A : Set l} {x y : A}
  → (x ≡ y) → y ≡ x
sym {l} {A} {x} {y} p = J (λ x y p → y ≡ x) (λ x → refl) x y p
```

### Lemma 2.1.2

```
trans : {l : Level} {A : Set l} {x y z : A}
  → (x ≡ y)
  → (y ≡ z)
  → (x ≡ z)
trans {l} {A} {x} {y} {z} p q =
  let
    func1 x y p = J (λ a b p → a ≡ b) (λ x → refl) x y p
    func2 = J (λ a b p → (z : A) → (q : b ≡ z) → a ≡ z) func1 x y p
  in func2 z q

infixr 30 _∙_
_∙_ = trans
```

### Equational Reasoning

```
module ≡-Reasoning where
  infix 1 begin_
  begin_ : {A : Set} {x y : A} → (x ≡ y) → (x ≡ y)
  begin x = x

  _≡⟨⟩_ : {A : Set} (x {y} : A) → x ≡ y → x ≡ y
  _ ≡⟨⟩ x≡y = x≡y

  infixr 2 _≡⟨⟩_ step-≡
  step-≡ : {A : Set} (x {y z} : A) → y ≡ z → x ≡ y → x ≡ z
  step-≡ _ y≡z x≡y = trans x≡y y≡z
  syntax step-≡  x y≡z x≡y = x ≡⟨  x≡y ⟩ y≡z

  infix 3 _∎
  _∎ : {A : Set} (x : A) → (x ≡ x)
  _ ∎ = refl
```

### Lemma 2.1.4

```
module lemma2∙1∙4 {A : Set} where
  i1 : {x y : A} (p : x ≡ y) → p ≡ p ∙ refl
  i1 {x} {y} p = J (λ x y p → p ≡ p ∙ refl) (λ _ → refl) x y p

  i2 : {x y : A} (p : x ≡ y) → p ≡ refl ∙ p
  i2 {x} {y} p = J (λ x y p → p ≡ refl ∙ p) (λ _ → refl) x y p

  ii1 : {x y : A} (p : x ≡ y) → (sym p) ∙ p ≡ refl
  ii1 {x} {y} p = J (λ x y p → (sym p) ∙ p ≡ refl) (λ _ → refl) x y p

  ii2 : {x y : A} (p : x ≡ y) → p ∙ (sym p) ≡ refl
  ii2 {x} {y} p = J (λ x y p → p ∙ (sym p) ≡ refl) (λ _ → refl) x y p

  iii : {x y : A} (p : x ≡ y) → sym (sym p) ≡ p
  iii {x} {y} p = J (λ x y p → sym (sym p) ≡ p) (λ _ → refl) x y p

  iv : {x y z w : A} (p : x ≡ y) (q : y ≡ z) (r : z ≡ w) → p ∙ (q ∙ r) ≡ (p ∙ q) ∙ r
  iv {x} {y} {z} {w} p q r = let
      open ≡-Reasoning

      C : (x y : A) → (p : x ≡ y) → Set
      C x y p = (q : y ≡ z) → p ∙ (q ∙ r) ≡ (p ∙ q) ∙ r

      c : (x : A) → C x x refl
      c x q1 = begin
        refl ∙ (q1 ∙ r) ≡⟨ sym (i2 (q1 ∙ r)) ⟩
        (q1 ∙ r) ≡⟨ ap (λ p → p ∙ r) (i2 q1) ⟩
        (refl ∙ q1) ∙ r ∎
    in J C c x y p q
```

### Definition 2.1.7 (pointed type)

This comes first, since it is needed to define Theorem 2.1.6.

```
record pointed (l : Level) : Set (lsuc l) where
  eta-equality
  constructor mkPointed
  field
    A : Set l
    a : A
```

### Definition 2.1.8 (loop space)

```
Ω : {l : Level} → pointed l → pointed l
Ω (mkPointed A a) = mkPointed (a ≡ a) refl
```

### Theorem 2.1.6 (Eckmann-Hilton)

TODO

```
Ω² : {l : Level} → pointed l → pointed l
Ω² p = Ω (Ω p)
```

## 2.2 Functions are functors

### Lemma 2.2.1

{{#include HottBook.Chapter2Lemma221.md:ap}}

### Lemma 2.2.2

Ap rules

```
module lemma2∙2∙2 {A B C : Set} where
  open ≡-Reasoning

  i : {f : A → B} {x y z : A} (p : x ≡ y) (q : y ≡ z) → ap f (p ∙ q) ≡ ap f p ∙ ap f q
  i {f} {x} {y} {z} p q = J
    ((λ x1 y1 p1 → (q1 : y1 ≡ z) → ap f (p1 ∙ q1) ≡ ap f p1 ∙ ap f q1))
    (λ x1 q1 → J
      (λ x2 y2 p2 → ap f (refl ∙ p2) ≡ ap f refl ∙ ap f p2)
      (λ x2 → refl) x1 z q1
    ) x y p q

  ii : {f : A → B} {x y : A} (p : x ≡ y) → ap f (sym p) ≡ sym (ap f p)
  ii {f} {x} {y} p = J (λ x1 y1 p1 → ap f (sym p1) ≡ sym (ap f p1)) (λ x1 → refl) x y p

  iii : (f : A → B) (g : B → C) {x y : A} (p : x ≡ y) → ap g (ap f p) ≡ ap (g ∘ f) p
  iii f g {x} {y} p = J (λ x1 y1 p1 → ap g (ap f p1) ≡  ap (g ∘ f) p1) (λ x1 → refl) x y p

  iv : {x y : A} (p : x ≡ y) → ap id p ≡ p
  iv {x} {y} p = J (λ x1 y1 p1 → ap id p1 ≡ p1) (λ x1 → refl) x y p
```

## 2.3 Type families are fibrations

### Lemma 2.3.1 (Transport)

```
transport : {l₁ l₂ : Level} {A : Set l₁} {x y : A}
  → (P : A → Set l₂)
  → (p : x ≡ y)
  → P x → P y
transport {l₁} {l₂} {A} {x} {y} P p = J (λ x y p → P x → P y) (λ x → id) x y p
```

### Lemma 2.3.2 (Path lifting property)

```
lift : {A : Set} {P : A → Set} {x y : A}
  → (u : P x)
  → (p : x ≡ y)
  → (x , u) ≡ (y , transport P p u)
lift {A} {P} {x} {y} u p =
  J (λ x1 y1 p1 → (u1 : P x1) → (x1 , u1) ≡ (y1 , transport P p1 u1)) (λ x1 u1 → refl) x y p u
```

Verifying its property:

```
module lemma2∙3∙2 where
  pr₁ : {A : Set} {P : A → Set}
    {x y : A}
    {u : P x} {v : P y}
    (p : (x , u) ≡ (y , v))
    → x ≡ y
  pr₁ p = ap (λ (x , y) → x) p

  open ≡-Reasoning

  lift-prop : {A : Set} {P : A → Set} {x y : A} (u : P x) → (p : x ≡ y)
    → pr₁ (lift {A} {P} u p) ≡ p
  lift-prop {A} {P} {x} {y} u p =
    J (λ x1 y1 p1 → (u1 : P x1) → pr₁ (lift {A} {P} u1 p1) ≡ p1) (λ x1 u1 → refl) x y p u
```

### Lemma 2.3.4 (Dependent map)

```
apd : {l₁ l₂ : Level} {A : Set l₁} {P : A → Set l₂} {x y : A}
  → (f : (x : A) → P x)
  → (p : x ≡ y)
  → transport P p (f x) ≡ f y
apd {l₁} {l₂} {A} {P} {x} {y} f p =
  J (λ x y p → transport P p (f x) ≡ f y) (λ x → refl) x y p
```

### Lemma 2.3.5

```
transportconst : {l₁ l₂ : Level} {A : Set l₁} {x y : A}
  → (B : Set l₂)
  → (p : x ≡ y)
  → (b : B)
  → transport (λ _ → B) p b ≡ b
transportconst {l₁} {l₂} {A} {x} {y} B p b =
  J (λ x y p → transport (λ _ → B) p b ≡ b) (λ x → refl) x y p
```

### Lemma 2.3.8

```
lemma2∙3∙8 : {l : Level} {A B : Set l}
  → (f : A → B) 
  → {x y : A} 
  → (p : x ≡ y)
  → apd f p ≡ transportconst B p (f x) ∙ ap f p
lemma2∙3∙8 {l} {A} {B} f {x} {y} p =
  J (λ x y p → apd f p ≡ transportconst B p (f x) ∙ ap f p) (λ x → refl) x y p
```

### Lemma 2.3.9

```
lemma2∙3∙9 : {A : Set}
  → {x y z : A}
  → (P : A → Set)
  → (p : x ≡ y) → (q : y ≡ z) → (u : P x)
  → transport P q (transport P p u) ≡ transport P (p ∙ q) u
lemma2∙3∙9 {A} {x} {y} {z} P p q u = 
  let 
    f2 : (x1 : A) → (q1 : x1 ≡ z) (u1 : P x1) → transport P q1 u1 ≡ transport P (refl ∙ q1) u1
    f2 x1 q1 u1 =
      ap (λ x → transport P x u1) (lemma2∙1∙4.i2 q1)
    f1 = J (λ x1 y1 p1 → (q1 : y1 ≡ z) → (u1 : P x1) → transport P q1 (transport P p1 u1) ≡ transport P (p1 ∙ q1) u1) f2 x y p
  in f1 q u
```

### Lemma 2.3.10

```
-- lemma2310 : {A B : Set} (f : A → B) → (P : B → Set)
--   → {x y : A} (p : x ≡ y) → (u : P (f x))
--   → transport (P ∘ f) p u ≡ transport P (ap f p) u
-- lemma2310 f P p u =
--   let
--     inner : transport (λ y → P (f y)) refl u ≡ u
--     inner = refl
--   in
--   J (λ _ what → transport (P ∘ f) what u ≡ transport P (ap f what) u) p inner
```

### Lemma 2.3.11

```
-- lemma2∙3∙11 : {A : Set} {P Q : A → Set}
--   → (f : (x : A) → P x → Q x)
--   → {x y : A} (p : x ≡ y) → (u : P x)
--   → transport Q p (f x u) ≡ f y (transport P p u)
-- lemma2∙3∙11 {A} {P} {Q} f {x} {y} p u =
--   J (λ a b p → transport Q p (f a {!   !}) ≡ f b (transport P p {!   !})) (λ a → {!   !}) x y p
  -- J (λ the what → transport Q what (f x u) ≡ f the (transport P what u)) p refl
```

## 2.4 Homotopies and equivalences

### Definition 2.4.1 (Homotopy)

```
infixl 18 _∼_
_∼_ : {l₁ l₂ : Level} {A : Set l₁} {P : A → Set l₂}
  → (f g : (x : A) → P x)
  → Set (l₁ ⊔ l₂)
_∼_ {l₁} {l₂} {A} {P} f g = (x : A) → f x ≡ g x
```

### Lemma 2.4.2

Homotopy forms an equivalence relation:

```
∼-refl : {A : Set} {P : A → Set}
  → (f : (x : A) → P x)
  → f ∼ f
∼-refl f x = refl

∼-sym : {A : Set} {P : A → Set}
  → {f g : (x : A) → P x}
  → f ∼ g → g ∼ f
∼-sym {f} {g} f∼g x = sym (f∼g x)

∼-trans : {l : Level} {A : Set l} {P : A → Set l}
  → {f g h : (x : A) → P x}
  → f ∼ g
  → g ∼ h
  → f ∼ h
∼-trans {l} {A} {P} {f} {g} {h} f∼g g∼h x = f∼g x ∙ g∼h x
```

### Lemma 2.4.3

```
lemma2∙4∙3 : {A B : Set} {f g : A → B} (H : f ∼ g) {x y : A} (p : x ≡ y)
  → H x ∙ ap g p ≡ ap f p ∙ H y
lemma2∙4∙3 {A} {B} {f} {g} H {x} {y} p = let
    open ≡-Reasoning
  in
  J (λ x1 y1 p1 → (H x1 ∙ ap g p1 ≡ ap f p1 ∙ H y1)) (λ x1 → begin
    H x1 ∙ ap g refl ≡⟨⟩
    H x1 ∙ refl ≡⟨ sym (lemma2∙1∙4.i1 (H x1)) ⟩
    H x1 ≡⟨ lemma2∙1∙4.i2 (H x1) ⟩
    refl ∙ H x1 ≡⟨⟩
    ap f refl ∙ H x1 ∎
  ) x y p
```

### Corollary 2.4.4

```
-- lemma2∙4∙4 : {A : Set} {f : A → A} {H : f ∼ id}
--   → (x : A)
--   → H (f x) ≡ ap f (H x)
-- lemma2∙4∙4 {A} {f} {H} x =
--   let g p = H x in
--   {!   !}
```

### Definition 2.4.6

```
record qinv {l : Level} {A B : Set l} (f : A → B) : Set l where
  constructor mkQinv
  field
    g : B → A
    α : (f ∘ g) ∼ id
    β : (g ∘ f) ∼ id
```

### Example 2.4.7

```
id-qinv : {l : Level} {A : Set l} → qinv {l} {A} id
id-qinv = mkQinv id (λ _ → refl) (λ _ → refl)
```

### Example 2.4.8

```

```

### Example 2.4.9

```
transport-qinv : {A : Set}
  → {x y : A}
  → (P : A → Set)
  → (p : x ≡ y)
  → qinv (transport P p)
transport-qinv P p = mkQinv (transport P (sym p))
  (λ py → 
    let wtf = lemma2∙3∙9 P (sym p) p py in
    trans wtf (ap (λ x → transport P x py) (lemma2∙1∙4.ii1 p))) 
  (λ px → 
    let wtf = lemma2∙3∙9 P p (sym p) px in
    trans wtf (ap (λ x → transport P x px) (lemma2∙1∙4.ii2 p))) 
```

### Definition 2.4.10

```
record isequiv {l : Level} {A B : Set l} (f : A → B) : Set l where
  eta-equality
  constructor mkIsEquiv
  field
    g : B → A
    g-id : (f ∘ g) ∼ id
    h : B → A
    h-id : (h ∘ f) ∼ id
```

```
qinv-to-isequiv : {l : Level} {A B : Set l}
  → {f : A → B}
  → qinv f
  → isequiv f
qinv-to-isequiv (mkQinv g α β) = mkIsEquiv g α g β
```

Still kinda shaky on this one, TODO study it later:

```
isequiv-to-qinv : {l : Level} {A B : Set l}
  → {f : A → B}
  → isequiv f
  → qinv f
isequiv-to-qinv {l} {A} {B} {f} (mkIsEquiv g g-id h h-id) =
  let
    γ : g ∼ h
    γ x = (sym (h-id (g x))) ∙ ap h (g-id x)

    β : (g ∘ f) ∼ id
    β x = γ (f x) ∙ h-id x 
  in
  mkQinv g g-id β
```

### Definition 2.4.11

```
_≃_ : {l : Level} → (A B : Set l) → Set l
A ≃ B = Σ[ f ∈ (A → B) ] (isequiv f)
```

### Lemma 2.4.12

```
module lemma2∙4∙12 where
  id-equiv : (A : Set) → A ≃ A
  id-equiv A = id , e
    where
      e : isequiv id
      e = mkIsEquiv id (λ _ → refl) id (λ _ → refl)

  sym-equiv : {A B : Set} → (f : A ≃ B) → B ≃ A
  sym-equiv {A} {B} (f , eqv) =
    let (mkQinv g α β) = isequiv-to-qinv eqv
    in g , qinv-to-isequiv (mkQinv f β α)

  trans-equiv : {A B C : Set} → (f : A ≃ B) → (g : B ≃ C) → A ≃ C
  trans-equiv {A} {B} {C} (f , f-eqv) (g , g-eqv) =
    let
      (mkQinv f-inv f-inv-left f-inv-right) = isequiv-to-qinv f-eqv
      (mkQinv g-inv g-inv-left g-inv-right) = isequiv-to-qinv g-eqv

      open ≡-Reasoning

      forward : ((g ∘ f) ∘ (f-inv ∘ g-inv)) ∼ id
      forward c =
        begin
          ((g ∘ f) ∘ (f-inv ∘ g-inv)) c ≡⟨ ap (λ f → f c) (composite-assoc (f-inv ∘ g-inv) f g) ⟩
          (g ∘ (f ∘ f-inv) ∘ g-inv) c ≡⟨ ap g (f-inv-left (g-inv c)) ⟩
          (g ∘ id ∘ g-inv) c ≡⟨⟩
          (g ∘ g-inv) c ≡⟨ g-inv-left c ⟩
          id c ∎

      backward : ((f-inv ∘ g-inv) ∘ (g ∘ f)) ∼ id
      backward a =
        begin
          ((f-inv ∘ g-inv) ∘ (g ∘ f)) a ≡⟨ ap (λ f → f a) (composite-assoc (g ∘ f) g-inv f-inv) ⟩
          (f-inv ∘ (g-inv ∘ (g ∘ f))) a ≡⟨ ap f-inv (g-inv-right (f a)) ⟩
          (f-inv ∘ (id ∘ f)) a ≡⟨⟩
          (f-inv ∘ f) a ≡⟨ f-inv-right a ⟩
          id a ∎
      
    in g ∘ f , qinv-to-isequiv (mkQinv (f-inv ∘ g-inv) forward backward)
```

## 2.7 Σ-types

### Theorem 2.7.2

_Suppose that P : A → U is a type family over a type A and let w, w′ : ∑(x:A) P(x). Then there is an equivalence_

```
theorem2∙7∙2 : {l : Level} {A : Set l} {P : A → Set l}
  → ((w @ (w1 , w2)) (w' @ (w'1 , w'2)) : Σ A P)
  → (w ≡ w') ≃ Σ (w1 ≡ w'1) (λ p → transport P p w2 ≡ w'2)
theorem2∙7∙2 {l} {A} {P} (w @ (w1 , w2)) (w' @ (w'1 , w'2)) =
  f , qinv-to-isequiv (mkQinv g forwards backwards)
  where
    f : (w ≡ w') → Σ (w1 ≡ w'1) (λ p → transport P p w2 ≡ w'2)
    f refl = refl , refl

    g : Σ (w1 ≡ w'1) (λ p → transport P p w2 ≡ w'2) → (w ≡ w') 
    g (refl , refl) = refl

    forwards : (f ∘ g) ∼ id
    forwards (refl , refl) = refl

    backwards : (g ∘ f) ∼ id
    backwards refl = refl
```

### Corollary 2.7.3

_For z : ∑(x:A) P(x), we have z = (pr1(z), pr2(z))._

```
corollary2∙7∙3 : {A : Set} {P : A → Set}
  → (z @ (a , b) : Σ A P)
  → z ≡ (a , b)
corollary2∙7∙3 z = refl
```

## 2.8 The unit type

### Theorem 2.8.1

```
theorem2∙8∙1 : (x y : 𝟙) → (x ≡ y) ≃ 𝟙
theorem2∙8∙1 x y = func x y , equiv x y
  where
    func : (x y : 𝟙) → (x ≡ y) → 𝟙
    func x y _ = tt

    rev : (x y : 𝟙) → 𝟙 → (x ≡ y)
    rev tt tt _ = refl

    backwards : (x y : 𝟙) → (func x y ∘ rev x y) ∼ id
    backwards tt tt tt = refl

    forwards : (x y : 𝟙) → (rev x y ∘ func x y) ∼ id
    forwards tt tt refl = refl

    equiv : (x y : 𝟙) → isequiv (func x y)
    equiv x y = record
      { g = rev x y
      ; g-id = backwards x y
      ; h = rev x y
      ; h-id = forwards x y
      }
```

## 2.9 Π-types and the function extensionality axiom

### Lemma 2.9.1

```
postulate
  lemma2∙9∙1 : {A : Set} {B : A → Set}
    → (f g : (x : A) → B x)
    → (f ≡ g) ≃ ((x : A) → f x ≡ g x)
```

### Lemma 2.9.2

```
happly : {A B : Set}
  → (f g : A → B)
  → {x : A} 
  → (p : f ≡ g)
  → f x ≡ g x
happly {A} {B} f g {x} p = ap (λ h → h x) p
```

### Axiom 2.9.3 (Function extensionality)

```
postulate
  funext : {l : Level} {A B : Set l}
    → {f g : A → B}
    → ((x : A) → f x ≡ g x)
    → f ≡ g
```

### Lemma 2.9.6

```
-- lemma2∙9∙6 : {X : Set} {A B : X → Set} {x y : X}
--   → (p : x ≡ y)
--   → (f : A x → B x)
--   → (g : A y → B y)
--   → 
--     let P x = A x → B x in
--     (transport P p f ≡ g) ≃ ((a : A x) → transport B p (f a) ≡ g (transport A p a))
-- lemma2∙9∙6 {X} {A} {B} {x} {y} p f g =
--   let
--     P x = A x → B x

--     F : (x1 : X) → (f1 : A x1 → B x1) → (g1 : A x1 → B x1) → (transport P refl f1 ≡ g1) → ((a : A x1) → transport B refl (f1 a) ≡ g1 (transport A refl a))
--     F x f g p a = {! !}
--   in
--   J (λ x1 y1 p1 → (f1 : A x1 → B x1) → (g1 : A y1 → B y1) → (transport P p1 f1 ≡ g1) ≃ ((a : A x1) → transport B p1 (f1 a) ≡ g1 (transport A p1 a))) 
--     (λ x1 f1 g1 → F x1 f1 g1 , qinv-to-isequiv (mkQinv {!   !} {!   !} {!   !})) x y p f g
```

## 2.10 Universes and the univalence axiom

### Lemma 2.10.1

```
idtoeqv : {l : Level} {A B : Set l}
  → (A ≡ B)
  → (A ≃ B)
idtoeqv {l} {A} {B} p = J C c A B p
  where
    C : (x y : Set l) (p : x ≡ y) → Set l
    C x y p = x ≃ y

    c : (x : Set l) → C x x refl
    c x = id , qinv-to-isequiv id-qinv
```

### Axiom 2.10.3 (Univalence)

```
postulate
  ua : {A B : Set} (eqv : A ≃ B) → (A ≡ B)
```

### Lemma 2.10.5

```
-- lemma2∙10∙5 : {l : Level} {A : Set l} {B : A → Set l}
--   → (x y : A)
--   → (p : x ≡ y)
--   → (u : B x)
--   → transport B p u ≡ transport id (ap B p) u
```

## 2.11 Identity Type

### Theorem 2.11.1

```
-- theorem2∙11∙1 : {A B : Set}
--   → (eqv @ (f , f-eqv) : A ≃ B)
--   → (a a' : A)
--   → (a ≡ a') ≃ (f a ≡ f a')
-- theorem2∙11∙1 (f , f-eqv) a a' =
--   let
--     open ≡-Reasoning
--     mkQinv g α β = isequiv-to-qinv f-eqv
--     inv : (f a ≡ f a') → a ≡ a'
--     inv p = (sym (β a)) ∙ (ap g p) ∙ (β a')
--     backward : (p : f a ≡ f a') → (ap f ∘ inv) p ≡ id p
--     backward q = begin
--       ap f ((sym (β a)) ∙ (ap g q) ∙ (β a')) ≡⟨ lemma2∙2∙2.i (sym (β a)) (ap g q ∙ β a')  ⟩
--       ap f (sym (β a)) ∙ ap f ((ap g q) ∙ (β a')) ≡⟨ {!   !} ⟩
--       ap f (sym (β a)) ∙ ap f ((ap g q) ∙ (β a')) ≡⟨ {!   !} ⟩
--       id q ∎
--     forward : (p : a ≡ a') → (inv ∘ ap f) p ≡ id p
--     forward p = begin
--       (sym (β a)) ∙ (ap g (ap f p)) ∙ (β a') ≡⟨ ap (λ p → (sym (β a)) ∙ p ∙ (β a')) (lemma2∙2∙2.iii f g p) ⟩
--       (sym (β a)) ∙ (ap (g ∘ f) p) ∙ (β a') ≡⟨ {!   !} ⟩
--       (sym (β a)) ∙ (ap id p) ∙ (β a') ≡⟨ {!   !} ⟩
--       id p ∎
--     eqv = mkQinv inv backward forward
--   in
--   ap f , qinv-to-isequiv eqv
```

## 2.13 Natural numbers

```
private
  code : ℕ → ℕ → Set
  code zero zero = 𝟙
  code zero (suc y) = ⊥
  code (suc x) zero = ⊥
  code (suc x) (suc y) = code x y
```

### Theorem 2.13.1

_For all $m, n : N$ we have $(m = n) \simeq \texttt{code}(m, n)$._

```text
theorem2∙13∙1 : (m n : ℕ) → (m ≡ n) ≃ code m n
theorem2∙13∙1 m n = encode m n , equiv
  where
    r : (n : ℕ) → code n n
    r zero = tt
    r (suc n) = r n

    encode : (m n : ℕ) → m ≡ n → code m n
    encode m n p = transport (λ n → code m n) p (r m)

    decode : (m n : ℕ) → code m n → m ≡ n
    decode zero zero c = refl
    decode (suc m) (suc n) c = ap suc (decode m n c)

    forward : (m n : ℕ) → (p : m ≡ n) → (encode m n ∘ decode m n) p ≡ id p
    forward m n p = J C c m n p
      where
        C : (x y : ℕ) → (p : x ≡ y) → Set
        C x y p = (encode x y ∘ decode x y) p ≡ id p

        c : (x : ℕ) → decode x x (r x) ≡ refl
        c zero = refl
        c (suc x) = ap (λ p → ap suc p) (c x)

    backward : (m n : ℕ) → (c : code m n) → (decode m n ∘ encode m n) c ≡ id c
    backward zero zero c =
      let
        what = encode zero zero refl
        what2 = decode zero zero c
        what3 = theorem2∙8∙1 what c
        what4 = isequiv.g (Σ.snd what3)
        what5 = what4 tt
      in what5
    backward (suc m) (suc n) c =
      let
        IH = backward m n c

        -- what : code m n
        -- what = encode (suc m) (suc n) (ap suc (decode m n c))
        -- what = transport (λ n → code (suc m) n) (ap suc (decode m n c)) (r (suc m))
        -- what = transport (λ n → code (suc m) (suc n)) (decode m n c) (r (suc m))
        -- what = transport (λ n → code m n) (decode m n c) (r m)
        -- what = transport (λ n → code m n) (decode m n c) (r m)
        -- what = transport (λ n → code m n) (decode m n c) (r m)

        res2 : transport (λ x → code (suc m) (suc x)) (decode m n c) (r (suc m)) ≡ c
        res2 = IH

        -- res : encode (suc m) (suc n) (ap suc (decode m n c)) ≡ c
        res : transport (λ n → code (suc m) n) (ap suc (decode m n c)) (r (suc m)) ≡ c
        res =     

      in res

    equiv = record
      { g = decode m n
      ; g-id = forward m n
      ; h = decode m n
      ; h-id = backward m n
      }
```

## 2.15 Universal properties

Definition 2.15.1

```
definition2∙15∙1 : {X A B : Set}
  → (X → A × B)
  → (X → A) × (X → B)
definition2∙15∙1 f = Σ.fst ∘ f , Σ.snd ∘ f
```

### Theorem 2.15.2

```
theorem2∙15∙2 : {X A B : Set} → isequiv (definition2∙15∙1 {X} {A} {B})
theorem2∙15∙2 {X} {A} {B} = mkIsEquiv g (λ _ → refl) g (λ _ → refl)
  where
    g : (X → A) × (X → B) → (X → A × B)
    g (f1 , f2) = λ x → (f1 x , f2 x)
```    