```
module HottBook.Chapter2Exercises where

open import Agda.Primitive
open import HottBook.Chapter1
open import HottBook.Chapter2
open import HottBook.Chapter2Lemma221
```

## Exercise 2.1

Show that the three obvious proofs of Lemma 2.1.2 are pair- wise equal.

```
module exercise2∙1 {l : Level} {A : Set l} where
  prf1 : {x y z : A} → (x ≡ y) → (y ≡ z) → (x ≡ z)
  prf1 refl q = q

  prf2 : {x y z : A} → (x ≡ y) → (y ≡ z) → (x ≡ z)
  prf2 p refl = p

  prf3 : {x y z : A} → (x ≡ y) → (y ≡ z) → (x ≡ z)
  prf3 refl refl = refl

  prf1≡prf2 : {x y z : A} → prf1 ≡ prf2
  prf1≡prf2 {x} {y} {z} =
    let
      ind p q = J
        (λ x1 y1 p1 → (q1 : y1 ≡ z) → prf1 p1 q1 ≡ prf2 {x1} {y1} {z} p1 q1)
        (λ x1 q1 → J (λ x1 z1 q1 → q1 ≡ prf2 refl q1) (λ _ → refl) x1 z q1)
        x y p q
    in
    funext (λ p → funext λ q → ind p q)

  prf2≡prf3 : {x y z : A} → prf2 ≡ prf3
  prf2≡prf3 {x} {y} {z} =
    let
      ind p q = J
        (λ x1 y1 p1 → (q1 : y1 ≡ z) → prf2 p1 q1 ≡ prf3 {x1} {y1} {z} p1 q1)
        (λ x1 q1 → J (λ x1 z1 q1 → prf2 refl q1 ≡ prf3 refl q1) (λ _ → refl) x1 z q1)
        x y p q
    in
    funext (λ p → funext λ q → ind p q)

  prf1≡prf3 : {x y z : A} → prf1 ≡ prf3
  prf1≡prf3 {x} {y} {z} =
    let
      ind p q = J
        (λ x1 y1 p1 → (q1 : y1 ≡ z) → prf1 p1 q1 ≡ prf3 {x1} {y1} {z} p1 q1)
        (λ x1 q1 → J (λ x1 z1 q1 → q1 ≡ prf3 refl q1) (λ _ → refl) x1 z q1)
        x y p q
    in
    funext (λ p → funext λ q → ind p q)
```

## Exercise 2.4

_Define, by induction on n, a general notion of n-dimensional path in a type
$A$, simultaneously with the type of boundaries for such paths._

(tracked in [#6][6])

[6]: https://git.mzhang.io/school/type-theory/issues/6

```

```

## Exercise 2.5

Prove that the functions [2.3.6] and [2.3.7] are inverse equivalences.

[2.3.6]: https://hott.github.io/book/hott-ebook-1357-gbe0b8e2.pdf#equation.2.3.6
[2.3.7]: https://hott.github.io/book/hott-ebook-1357-gbe0b8e2.pdf#equation.2.3.7

## Exercise 2.13

_Show that $(2 \simeq 2) \simeq 2$._

(tracked in [#8][8])

[8]: https://git.mzhang.io/school/type-theory/issues/8

lemma: equivalences are equal if their functions are equal

```
postulate
  posEqLemma : {A B : Set}
    → (e1 e2 : A ≃ B)
    → (Σ.fst e1 ≡ Σ.fst e2)
    → e1 ≡ e2

-- What the hell
-- https://agda.readthedocs.io/en/v2.6.1/language/with-abstraction.html#the-inspect-idiom
module Wtf {a b} {A : Set a} {B : A → Set b} where
  data Graph (f : ∀ x → B x) (x : A) (y : B x) : Set b where
    ingraph : f x ≡ y → Graph f x y
  inspect : (f : ∀ x → B x) (x : A) → Graph f x (f x)
  inspect _ _ = ingraph refl
open Wtf
```

main proof:

```
exercise2∙13 : (𝟚 ≃ 𝟚) ≃ 𝟚
exercise2∙13 = f , equiv
  where
    neg : 𝟚 → 𝟚
    neg true = false
    neg false = true

    f : 𝟚 ≃ 𝟚 → 𝟚
    f (fst , snd) = fst true

    id-equiv : isequiv id
    id-equiv = mkIsEquiv id (λ _ → refl) id (λ _ → refl)

    neg-id : (neg ∘ neg) ∼ id
    neg-id true = refl
    neg-id false = refl

    neg-equiv : isequiv neg
    neg-equiv = mkIsEquiv neg neg-id neg neg-id

    g : 𝟚 → 𝟚 ≃ 𝟚
    g true = id , id-equiv
    g false = neg , neg-equiv

    f∘g∼id : (f ∘ g) ∼ id
    f∘g∼id true = refl
    f∘g∼id false = refl

    g∘f∼id : (g ∘ f) ∼ id
    g∘f∼id e' @ (f' , ie' @ (mkIsEquiv g' g-id h' h-id)) = attempt
      where
        Bmap : 𝟚 → Set
        Bmap true = 𝟙
        Bmap false = ⊥

        true≢false : true ≢ false
        true≢false p =
          let genBot = transport Bmap p in
          genBot tt

        f-codomain-is-2 : f' true ≢ f' false
        f-codomain-is-2 p =
          let p1 = ap h' p in
          let p2 = trans p1 (h-id false) in
          let p3 = trans (sym (h-id true)) p2 in
          true≢false p3

        ⊥-elim : {A : Set} → ⊥ → A
        ⊥-elim ()

        opposite-prop : {a b : 𝟚} → (p : f' a ≡ b) → f' (neg a) ≡ neg b
        opposite-prop {a} {b} p with f' (neg a) | inspect f' (neg a)
        opposite-prop {true} {true} p | true | ingraph q = ⊥-elim (f-codomain-is-2 (trans p (sym q)))
        opposite-prop {true} {true} p | false | _ = refl
        opposite-prop {true} {false} p | true | _ = refl
        opposite-prop {true} {false} p | false | ingraph q = ⊥-elim (f-codomain-is-2 (trans p (sym q)))
        opposite-prop {false} {true} p | true | ingraph q = ⊥-elim (f-codomain-is-2 (trans q (sym p)))
        opposite-prop {false} {true} p | false | ingraph q = refl
        opposite-prop {false} {false} p | true | ingraph q = refl
        opposite-prop {false} {false} p | false | ingraph q = ⊥-elim (f-codomain-is-2 (trans q (sym p)))

        f-is-id' : (f' true ≡ true) → (b : 𝟚) → f' b ≡ id b
        f-is-id' p true = p
        f-is-id' p false = opposite-prop p

        f-is-id : (f' true ≡ true) → f' ≡ id
        f-is-id p = funext (f-is-id' p)

        f-is-neg' : (f' true ≡ false) → (b : 𝟚) → f' b ≡ neg b
        f-is-neg' p true = p
        f-is-neg' p false = opposite-prop p

        f-is-neg : (f' true ≡ false) → f' ≡ neg
        f-is-neg p = funext (f-is-neg' p)

        attempt : g (f' true) ≡ e'
        attempt with (f' true) | inspect f' true
        attempt | true | ingraph p = posEqLemma (id , id-equiv) e' (sym (f-is-id p))
        attempt | false | ingraph p = posEqLemma (neg , neg-equiv) e' (sym (f-is-neg p))

    equiv : isequiv f
    equiv = mkIsEquiv g f∘g∼id g g∘f∼id
```
