```
module HottBook.Chapter3 where

open import Agda.Primitive
open import HottBook.Chapter1
open import HottBook.Chapter1Util
open import HottBook.Chapter2
open import HottBook.Util

```

## 3.1 Sets and n-types

### Definition 3.1.1

```
isSet : {l : Level} (A : Set l) → Set l
isSet A = (x y : A) → (p q : x ≡ y) → p ≡ q
```

### Example 3.1.2

```
𝟙-is-Set : isSet 𝟙
𝟙-is-Set tt tt refl refl = refl
```

### Example 3.1.3

```
⊥-is-Set : isSet ⊥
⊥-is-Set () () p q
```

### Example 3.1.4

```
-- ℕ-is-Set : isSet ℕ
-- ℕ-is-Set = 
```

## 3.2 Propositions as types?

## 3.3 Mere propositions

### Definition 3.3.1

```
isProp : (P : Set) → Set
isProp P = (x y : P) → x ≡ y
```

### Lemma 3.3.2

```
-- lemma3∙2∙2 : {P : Set}
--   → isProp P
--   → (x₀ : P)
--   → P ≃ 𝟙
-- lemma3∙2∙2 {P} PisProp x₀ = f , mkIsEquiv g {!  forwards !} g {!   !}
--   where
--     f : P → 𝟙
--     f x = tt

--     g : 𝟙 → P
--     g u = x₀

--     forwards : f ∘ g ∼ id
--     forwards x = {! refl  !}
```

## 3.4 Classical vs. intuitionistic logic

### Definition 3.4.3

#### i. Decidability of types

```
-- decidable : (A : Set) → Set
-- decidable A = A ⊎ ¬ A
```

#### ii. Decidability of type families

```
-- dep-decidable : {A : Set} → (B : A → Set) → Set
-- dep-decidable {A} B = (a : A) → (B a ⊎ ¬ B a)
```

#### iii. Decidable equality

```
-- decidable-equality : (A : Set) → Set
-- decidable-equality A = (a b : A) → (a ≡ b) ⊎ ¬ (a ≡ b)
```

So I think the interesting property about this is that normally a function f
would return something of the type a ≡ b, which for any a and b would always
produce that one path. But there could exist other functions that produce
different paths, like (g : ... → a ≡ b).

What this (a ≡ b) ⊎ ¬ (a ≡ b) business is saying is: if f couldn't produce a
path, then **NO** other functions will ever be able to produce paths (along with
a proof that this is true). This means f is _exclusively_ the deciding factor
for whether a and b are equal under this type, and it's the only one that mints
paths for a ≡ b.

---

> 2. In case you are looking for advanced exercises, you can try to prove that
> any type with decidable equality is a set.

```
-- decidable-equality-is-set : (A : Set) → decidable-equality A → isSet A
-- decidable-equality-is-set A f x y p q =
--   let
--     res = f x y

--     -- We can eliminate the bottom case because if p and q exist, it must be inj₁
--     center : x ≡ y
--     center = [ (λ p′ → p′) , (λ p′ → ⊥-elim (p′ _)) ] (f x y)

--     -- What i want is: given any path x ≡ y, prove that it equals this inj₁ (res f x y)
--     guarantee : (p′ : x ≡ y) → center ≡ p′
--     guarantee p′ = J (λ the what → the ≡ p′) ? ?

--     info = ?
--   in
--   J (λ the what → p ≡ ?) refl ?
```
